Imports System.Xml

''' <summary>
''' Formats XML documents in a readable format
''' </summary>
''' <author>Mikael Levn</author>
''' <created>2007-12-10</created>
''' <version>1.0.1</version>
''' <remarks></remarks>
Public Module FormatXML


#Region "TabulationOptions class"
    ''' <summary>
    ''' The TabulationOptions class enables customization of the tabulated output
    ''' </summary>
    ''' <remarks></remarks>
    Public Class TabulationOptions

        Private _columns As System.Collections.Generic.List(Of ColumnItem)
        Private _dataQuery As String
        Private _useHeader As Boolean

        ''' <summary>
        ''' Creates a new TabulationOptions object and sets it data query
        ''' </summary>
        ''' <param name="DataQuery"></param>
        ''' <remarks></remarks>
        Public Sub New(ByVal DataQuery As String)
            _dataQuery = DataQuery
            _columns = New System.Collections.Generic.List(Of ColumnItem)
            _useHeader = False
        End Sub

        ''' <summary>
        ''' Adds a columns to be visible in the output data
        ''' </summary>
        ''' <param name="NodeName">The column/node name</param>
        ''' <param name="ColumnSpace">Number of tab characters to be inserted after (to the right of) the text</param>
        ''' <param name="Width">The total width (in characaters) the column should be, if the data i less than the lengt it will be padded</param>
        ''' <param name="Align">Sets in wich side the padding should be added</param>
        ''' <remarks></remarks>
        Public Sub AddColumn(ByVal NodeName As String, ByVal ColumnSpace As Integer, ByVal Width As Integer, Optional ByVal Align As ColumnAlign = ColumnAlign.Left)
            Dim columnItem As New ColumnItem()

            columnItem.Name = NodeName
            columnItem.ColumnHeader = String.Empty
            columnItem.PadLength = Width
            columnItem.ColumnSpace = ColumnSpace
            columnItem.Align = Align

            _columns.Add(columnItem)
        End Sub

        ''' <summary>
        ''' Adds a columns to be visible in the output data
        ''' </summary>
        ''' <param name="NodeName">The column/node name</param>
        ''' <param name="HeaderText">The columns header text</param>
        ''' <param name="ColumnSpace">Number of tab characters to be inserted after (to the right of) the text</param>
        ''' <param name="Width">The total width (in characaters) the column should be, if the data i less than the lengt it will be padded</param>
        ''' <param name="Align">Sets in wich side the padding should be added</param>
        ''' <remarks>The column's width will be automaticly extended if the header text is wider than the preferred width</remarks>
        Public Sub AddColumn(ByVal NodeName As String, ByVal HeaderText As String, ByVal ColumnSpace As Integer, ByVal Width As Integer, Optional ByVal Align As ColumnAlign = ColumnAlign.Left)
            Dim columnItem As New ColumnItem()

            If Not _useHeader Then
                _useHeader = True
            End If

            ' Increase the column width if it's to narrow to fit the headline
            If HeaderText.Length > Width Then
                Width = HeaderText.Length
            End If

            columnItem.Name = NodeName
            columnItem.ColumnHeader = HeaderText
            columnItem.PadLength = Width
            columnItem.ColumnSpace = ColumnSpace
            columnItem.Align = Align

            _columns.Add(columnItem)
        End Sub

        ''' <summary>
        ''' Gets the collection with columns
        ''' </summary>
        ''' <value></value>
        ''' <returns></returns>
        ''' <remarks></remarks>
        Public ReadOnly Property Columns() As System.Collections.Generic.List(Of ColumnItem)
            Get
                Return _columns
            End Get
        End Property

        ''' <summary>
        ''' Gets the data query text
        ''' </summary>
        ''' <value></value>
        ''' <returns></returns>
        ''' <remarks></remarks>
        Public ReadOnly Property DataQuery() As String
            Get
                Return _dataQuery
            End Get
        End Property



        ''' <summary>
        ''' Gets flag if header should be written
        ''' </summary>
        ''' <value></value>
        ''' <returns></returns>
        ''' <remarks></remarks>
        Public ReadOnly Property UseHeader() As Boolean
            Get
                Return _useHeader
            End Get
        End Property


        ''' <summary>
        ''' Enumeration to choose if text should be left or right aligned
        ''' </summary>
        ''' <remarks></remarks>
        Public Enum ColumnAlign
            Left
            Right
        End Enum

        ''' <summary>
        ''' Data structure to hold one column definition
        ''' </summary>
        ''' <remarks></remarks>
        Public Structure ColumnItem
            Public Name As String
            Public ColumnHeader As String
            Public ColumnSpace As Integer
            Public PadLength As Integer
            Public Align As ColumnAlign

        End Structure


    End Class

#End Region


#Region "IndentationOptions class"

    ''' <summary>
    ''' The IndentationOptions class enables customization of the indented output
    ''' </summary>
    ''' <remarks></remarks>
    Public Class IndentationOptions
        Private _omitXmlDeclaration As Boolean

        Public Sub New(ByVal OmitXmlDeclaration As Boolean)
            _omitXmlDeclaration = OmitXmlDeclaration
        End Sub
    End Class

#End Region


#Region "Shared tabulation methods"

    ''' <summary>
    ''' Converts a XML document to a more user friendly format where data is presented
    ''' in columns instead of a hierarchical XML structure
    ''' </summary>
    ''' <param name="fileName">The path to the XML file to be converted</param>
    ''' <param name="Options">The options object containing the customizations selected</param>
    ''' <returns>If successfull the methods returs a tabulated string containing the values read from the XML document</returns>
    ''' <remarks></remarks>
    Public Function TabulateFile(ByVal fileName As String, ByVal Options As TabulationOptions) As String
        Return TabulateXml(loadFileToDocument(fileName), Options)
    End Function

    ''' <summary>
    ''' Converts a XML document to a more user friendly format where data is presented
    ''' in columns instead of a hierarchical XML structure
    ''' </summary>
    ''' <param name="rawXml">The raw XML data to be used as datasource</param>
    ''' <param name="Options">The options object containing the customizations selected</param>
    ''' <returns>If successfull the methods returs a tabulated string containing the values read from the XML document</returns>
    ''' <remarks></remarks>
    Public Function TabulateXml(ByVal rawXml As String, ByVal Options As TabulationOptions) As String
        Return TabulateXml(loadRawXmlToDocument(rawXml), Options)
    End Function

    ''' <summary>
    ''' Converts a XML document to a more user friendly format where data is presented
    ''' in columns instead of a hierarchical XML structure
    ''' </summary>
    ''' <param name="xmlDocument">The XML document to be used as datasource</param>
    ''' <param name="Options">The options object containing the customizations selected</param>
    ''' <returns>If successfull the methods returs a tabulated string containing the values read from the XML document</returns>
    ''' <remarks></remarks>
    Public Function TabulateXml(ByVal xmlDocument As XmlDocument, ByVal Options As TabulationOptions) As String

        Dim output As New System.Text.StringBuilder
        Dim xmlData As XmlDataDocument = xmlDocument.Clone()
        Dim isHeaderWritten As Boolean = False

        ' Step throug all nodes matching the data query
        For Each Item As XmlNode In xmlData.SelectNodes(Options.DataQuery)

            ' Skip empty nodes
            If Item.ChildNodes.Count = 0 Then
                Continue For
            End If

            ' Write header (if enabled)
            If Not isHeaderWritten AndAlso Options.UseHeader Then

                ' Step trough the collection with columns
                For Each column As TabulationOptions.ColumnItem In Options.Columns
                    Dim data As String = column.ColumnHeader
                    Dim space As String = String.Empty

                    ' Add padding space if needed
                    If data.Length < column.PadLength Then
                        If column.Align = TabulationOptions.ColumnAlign.Left Then
                            data = data.PadRight(column.PadLength)
                        Else
                            data = data.PadLeft(column.PadLength)
                        End If
                    End If

                    ' Add columnspace
                    If column.ColumnSpace > 0 Then
                        space = String.Empty.PadLeft(column.ColumnSpace, Microsoft.VisualBasic.Chr(9))
                    End If

                    ' Append text to output
                    output.AppendFormat("{0}{1}", data, space)

                Next

                ' Append NewLine to end this row
                output.Append(Environment.NewLine)

                isHeaderWritten = True

            End If

            ' Step trough the collection with columns
            For Each column As TabulationOptions.ColumnItem In Options.Columns
                Dim columnNode As XmlNode = Item.SelectSingleNode(column.Name)

                If columnNode Is Nothing Then
                    Throw New NullReferenceException("Kunde inte hitta ett element med namnet '" + column.Name + "'")
                End If

                Dim data As String = columnNode.InnerText
                Dim space As String = String.Empty

                ' Add padding space if needed
                If data.Length < column.PadLength Then
                    If column.Align = TabulationOptions.ColumnAlign.Left Then
                        data = data.PadRight(column.PadLength)
                    Else
                        data = data.PadLeft(column.PadLength)
                    End If
                End If

                ' Add columnspace
                If column.ColumnSpace > 0 Then
                    space = String.Empty.PadLeft(column.ColumnSpace, Microsoft.VisualBasic.Chr(9))
                End If

                ' Append text to output
                output.AppendFormat("{0}{1}", data, space)

            Next

            ' Append NewLine to end this row
            output.Append(Environment.NewLine)

        Next

        ' Return the concatenated output
        Return output.ToString()

    End Function

#End Region


#Region "Shared indentation methods"

    ''' <summary>
    ''' Formats an XML data source to an indented and more readable format
    ''' </summary>
    ''' <param name="fileName">The path to the file to format</param>
    ''' <returns>If successful the method returns the formatted XML document as a string</returns>
    ''' <remarks></remarks>
    Public Function IndentFile(ByVal fileName As String, Optional ByVal OmitXmlDeclaration As Boolean = True) As String
        Return IndentXml(loadFileToDocument(fileName), OmitXmlDeclaration)
    End Function

    ''' <summary>
    ''' Formats an XML data source to an indented and more readable format
    ''' </summary>
    ''' <param name="rawXml">The raw XML data used as datasource</param>
    ''' <returns>If successful the method returns the formatted XML document as a string</returns>
    ''' <remarks></remarks>
    Public Function IndentXml(ByVal rawXml As String, Optional ByVal OmitXmlDeclaration As Boolean = True) As String
        Return IndentXml(loadRawXmlToDocument(rawXml), OmitXmlDeclaration)
    End Function

    ''' <summary>
    ''' Formats an XML data source to an indented and more readable format
    ''' </summary>
    ''' <param name="xmlDocument">The XML data source to be formatted</param>
    ''' <returns>If successful the method returns the formatted XML document as a string</returns>
    ''' <remarks></remarks>
    Public Function IndentXml(ByVal xmlDocument As XmlDocument, Optional ByVal OmitXmlDeclaration As Boolean = True) As String
        ' Create string writer used as a buffer to hold the formatted xml text 
        Dim xmlStringWriter As New System.IO.StringWriter()

        ' Create a cloned document to prevent unintentional updates or other reference issues
        Dim xmlDoc As XmlDataDocument
        xmlDoc = xmlDocument.Clone()

        ' Create settings object and set formatting mode to indent and (if choosen) omit xml declaration
        Dim xmlFormatterSettings As New XmlWriterSettings()
        xmlFormatterSettings.OmitXmlDeclaration = OmitXmlDeclaration
        xmlFormatterSettings.Indent = True

        ' Create the class that formats (indents) the xml document and 
        ' writes the output to our text writer 
        Dim formattedWriter As XmlWriter
        formattedWriter = XmlTextWriter.Create(xmlStringWriter, xmlFormatterSettings)

        ' Save the formatted document data from XmlDocument to the xml writer (and trough to our xml string writer) 
        xmlDoc.Save(formattedWriter)

        ' Free up temp resources
        xmlDoc = Nothing
        formattedWriter = Nothing
        xmlFormatterSettings = Nothing

        ' Return the formatted xml as a text 
        Return xmlStringWriter.ToString()
    End Function

#End Region


#Region "Private methods"

    ''' <summary>
    ''' Loads a raw XML strcutred string into a XML DOM document
    ''' </summary>
    ''' <param name="rawXml">Raw XML data strcuture to be loaded</param>
    ''' <returns>Returns a populated XmlDataDocument</returns>
    ''' <remarks></remarks>
    Private Function loadRawXmlToDocument(ByVal rawXml As String) As XmlDataDocument
        Dim document As New XmlDataDocument()

        If String.IsNullOrEmpty(rawXml) Then
            Throw New ArgumentNullException("Parametern RawXml r obligatorisk och mste innehlla ett vlformulerat XML-dokument")
        End If

        ' Load xml data into XmlDocument 
        document.LoadXml(rawXml)

        Return document
    End Function

    ''' <summary>
    ''' Loads a XML-file into an XML DOM document
    ''' </summary>
    ''' <param name="fileName">Path to the file to load</param>
    ''' <returns>Returns a populated XmlDataDocument</returns>
    ''' <remarks></remarks>
    Private Function loadFileToDocument(ByVal fileName As String) As XmlDataDocument
        Dim document As New XmlDataDocument()

        ' Load xml data into XmlDocument 
        document.Load(fileName)

        Return document
    End Function

#End Region


End Module
